@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -30,6 +30,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* Definition of program-specific options. */
 struct argp_option program_options[] =
   {
+    /* Input options. */
     {
       "globalhdu",
       UI_KEY_GLOBALHDU,
@@ -87,6 +88,7 @@ struct argp_option program_options[] =
 
 
 
+    /* Output options. */
     {
       "onedasimage",
       UI_KEY_ONEDASIMAGE,
@@ -152,6 +154,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "writeall",
+      UI_KEY_WRITEALL,
+      0,
+      0,
+      "Write all images in stack (not just a single one).",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->writeall,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
     {0}
   };
