@@ -5,7 +5,7 @@ Convolve is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -823,9 +823,13 @@ convolve(struct convolveparams *p)
      the output. */
   if( gal_fits_name_is_fits(p->cp.output) )
     {
-      gal_fits_key_write_filename("input", p->filename, &cp->okeys, 1);
+      gal_fits_key_write_filename("input", p->filename, &cp->okeys, 1,
+                                  cp->quiet);
       gal_fits_key_write_config(&cp->okeys, "Convolve configuration",
                                 "CONVOLVE-CONFIG", cp->output, "0");
     }
-  printf("  - Output: %s\n", p->cp.output);
+
+  /* Inform the user that the job is done. */
+  if(!p->cp.quiet)
+    printf("  - Output: %s\n", p->cp.output);
 }
