@@ -5,7 +5,7 @@ Match is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2021, Free Software Foundation, Inc.
+Copyright (C) 2017-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -156,6 +156,32 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       gal_options_parse_csv_float64
     },
+    {
+      "kdtree",
+      UI_KEY_KDTREE,
+      "STR",
+      0,
+      "build, internal, disable, CUSTOM-FITS-FILE.",
+      UI_GROUP_CATALOGMATCH,
+      &p->kdtree,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+    },
+    {
+      "kdtreehdu",
+      UI_KEY_KDTREEHDU,
+      "STR",
+      0,
+      "HDU of k-d tree when --kdtree=FILE.fits.",
+      UI_GROUP_CATALOGMATCH,
+      &p->kdtreehdu,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+    },
 
 
 
