@@ -5,7 +5,7 @@ Match is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2018, Free Software Foundation, Inc.
+Copyright (C) 2017-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -40,7 +40,7 @@ enum program_args_groups
 
 /* Available letters for short options:
 
-   b d e f g i j k m n p r s t u v w x y z
+   b e f g i j m n p r s t u v w x y z
    A B E G J L O Q R W X Y
 */
 enum option_keys_enum
@@ -51,11 +51,14 @@ enum option_keys_enum
   UI_KEY_LOGASOUTPUT     = 'l',
   UI_KEY_CCOL1           = 'c',
   UI_KEY_CCOL2           = 'C',
+  UI_KEY_COORD           = 'd',
+  UI_KEY_KDTREE          = 'k',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
   UI_KEY_NOTMATCHED      = 1000,
   UI_KEY_OUTCOLS,
+  UI_KEY_KDTREEHDU,
 };
 
 
