@@ -5,7 +5,7 @@ MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -144,7 +144,7 @@ saveindividual(struct mkonthread *mkp)
     {
       if( asprintf(&filename, "%s%zu_%s", outdir, ibq->id, p->basename)<0 )
         error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
-      gal_checkset_writable_remove(filename, 0, p->cp.dontdelete);
+      gal_checkset_writable_remove(filename, NULL, 0, p->cp.dontdelete);
     }
 
 
@@ -258,9 +258,9 @@ saveindividual(struct mkonthread *mkp)
       gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "MFORFLATPIX", 0,
                             &p->mforflatpix, 0, "Magnitude is flat pixel "
                             "value", 0, NULL, 0);
-  gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "MCOLISBRIGHTNESS", 0,
-                        &p->mcolisbrightness, 0, "Catalog's magnitude is "
-                        "actually brightness", 0, NULL, 0);
+  gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "MCOLISSUM", 0,
+                        &p->mcolissum, 0, "Catalog's magnitude is "
+                        "total sum", 0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "MAGATPEAK", 0,
                         &p->magatpeak, 0, "Magnitude is for peak pixel, "
                         "not full profile", 0, NULL, 0);
@@ -872,7 +872,7 @@ mkprof(struct mkprofparams *p)
       if( asprintf(&tmp, "Zeropoint: %g", p->zeropoint)<0 )
         error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_list_str_add(&comments, tmp, 0);
-      gal_checkset_writable_remove(LOGFILENAME, 0, p->cp.dontdelete);
+      gal_checkset_writable_remove(LOGFILENAME, NULL, 0, p->cp.dontdelete);
       gal_table_write_log(p->log, PROGRAM_STRING, &p->rawtime, comments,
                           LOGFILENAME, p->cp.quiet);
       gal_list_str_free(comments, 1);
