@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -44,19 +44,6 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
-    {
-      "refcol",
-      UI_KEY_REFCOL,
-      "STR",
-      0,
-      "Reference column name or number.",
-      GAL_OPTIONS_GROUP_INPUT,
-      &p->refcol,
-      GAL_TYPE_STRING,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
     {
       "greaterequal",
       UI_KEY_GREATEREQUAL,
@@ -251,6 +238,20 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_add_to_single_value
     },
+    {
+      "quantofmean",
+      UI_KEY_QUANTOFMEAN,
+      0,
+      0,
+      "Quantile of the mean.",
+      UI_GROUP_SINGLE_VALUE,
+      &p->singlevalue,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value
+    },
     {
       "mode",
       UI_KEY_MODE,
@@ -567,6 +568,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "outliernumngb",
+      UI_KEY_OUTLIERNUMNGB,
+      "INT",
+      0,
+      "Num neighboring tiles to look for outlier.",
+      UI_GROUP_SKY,
+      &p->outliernumngb,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "outliersclip",
       UI_KEY_OUTLIERSCLIP,
@@ -801,6 +815,108 @@ struct argp_option program_options[] =
     },
 
 
+    {
+      0, 0, 0, 0,
+      "Fitting (1 dimensional)",
+      UI_GROUP_FIT
+    },
+    {
+      "fit",
+      UI_KEY_FIT,
+      "STR",
+      0,
+      "Type of fitting: 'linear', 'linear-weighted', "
+      "'linear-no-constant', 'linear-no-constant-weighted', "
+      "'polynomial', 'polynomial-weighted', "
+      "'polynomial-robust'.",
+      UI_GROUP_FIT,
+      &p->fitname,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitweight",
+      UI_KEY_FITWEIGHT,
+      "STR",
+      0,
+      "Input weight: 'std', 'var' or 'inv-variance'.",
+      UI_GROUP_FIT,
+      &p->fitweight,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitmaxpower",
+      UI_KEY_FITMAXPOWER,
+      "INT",
+      0,
+      "Maximum power in polynomial to fit.",
+      UI_GROUP_FIT,
+      &p->fitmaxpower,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitrobust",
+      UI_KEY_FITROBUST,
+      "STR",
+      0,
+      "The robust function name to use.",
+      UI_GROUP_FIT,
+      &p->fitrobustname,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitestimate",
+      UI_KEY_FITESTIMATE,
+      "STR/FLT",
+      0,
+      "Estimate fitted func. (number, file or 'self').",
+      UI_GROUP_FIT,
+      &p->fitestimate,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitestimatehdu",
+      UI_KEY_FITESTIMATEHDU,
+      "STR/INT",
+      0,
+      "HDU containing the --fitestimate values.",
+      UI_GROUP_FIT,
+      &p->fitestimatehdu,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitestimatecol",
+      UI_KEY_FITESTIMATECOL,
+      "STR/INT",
+      0,
+      "Column containing the --fitestimate values.",
+      UI_GROUP_FIT,
+      &p->fitestimatecol,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
+
     {0}
   };
 
