@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -46,6 +46,15 @@ enum statistics_input_format
     INPUT_FORMAT_IMAGE,
   };
 
+enum statistics_fit_weight_types
+  {
+    STATISTICS_FIT_WHT_INVALID,
+
+    STATISTICS_FIT_WHT_STD,
+    STATISTICS_FIT_WHT_VAR,
+    STATISTICS_FIT_WHT_INVVAR,
+  };
+
 
 
 
@@ -59,7 +68,6 @@ struct statisticsparams
   gal_list_f64_t  *tp_args;  /* Arguments for printing.                  */
   char          *inputname;  /* Input filename.                          */
   gal_list_str_t  *columns;  /* Column name or number if input is table. */
-  char             *refcol;  /* Reference column name or number.         */
   float       greaterequal;  /* Only use values >= this value.           */
   float      greaterequal2;  /* Only use values >= this value (2D hist). */
   float           lessthan;  /* Only use values <  this value.           */
@@ -68,11 +76,18 @@ struct statisticsparams
   float           quantmax;  /* Quantile maximum.                        */
   uint8_t           ontile;  /* Do single value calculations on tiles.   */
   uint8_t      interpolate;  /* Use interpolation to fill blank tiles.   */
+  char            *fitname;  /* Name of fitting function to use.         */
+  char          *fitweight;  /* Input weight is 'std' or 'invvar'.       */
+  char        *fitestimate;  /* Values to estimate the fit.              */
+  char     *fitestimatecol;  /* Column for values to estimate the fit.   */
+  char     *fitestimatehdu;  /* HDU for values to estimate the fit.      */
+  size_t       fitmaxpower;  /* Maximum power of polynomial fit.         */
+  char      *fitrobustname;  /* Name of robust function.                 */
 
   uint8_t        asciihist;  /* Print an ASCII histogram.                */
   uint8_t         asciicfp;  /* Print an ASCII cumulative frequency plot.*/
   uint8_t        histogram;  /* Save histogram in output.                */
-  uint8_t      histogram2d;  /* Save 2D-histogram in output.             */
+  char        *histogram2d;  /* Save 2D-histogram as image or table.     */
   uint8_t       cumulative;  /* Save cumulative distibution in output.   */
   double            mirror;  /* Mirror value for hist and CFP.           */
   uint8_t              sky;  /* Find the Sky value over the image.       */
@@ -93,6 +108,7 @@ struct statisticsparams
   char         *kernelname;  /* File name of kernel to convolve input.   */
   char               *khdu;  /* Kernel HDU.                              */
   float       meanmedqdiff;  /* Mode and median quantile difference.     */
+  size_t     outliernumngb;  /* Number of neighbors to define outliers.  */
   float       outliersigma;  /* Multiple of sigma to define outlier.     */
   double   outliersclip[2];  /* Outlier Sigma-clipping params.           */
   size_t       smoothwidth;  /* Width of flat kernel to smooth interpd.  */
@@ -107,7 +123,6 @@ struct statisticsparams
   uint8_t        needssort;  /* If sorting is needed.                    */
   gal_data_t        *input;  /* Input data structure.                    */
   gal_data_t       *sorted;  /* Sorted input data structure.             */
-  gal_data_t    *reference;  /* Reference data structure.                */
   int               isfits;  /* Input is a FITS file.                    */
   int             hdu_type;  /* Type of HDU (image or table).            */
   gal_data_t       *kernel;  /* Kernel for convolution of input for Sky. */
@@ -116,6 +131,10 @@ struct statisticsparams
   gal_data_t        *std_t;  /* Sky standard deviation on each tile.     */
   char       *checkskyname;  /* Name of file for Sky calculation steps.  */
   time_t           rawtime;  /* Starting time of the program.            */
+  uint8_t            fitid;  /* ID of desired fit.                       */
+  uint8_t      fitrobustid;  /* ID of robust fit type.                   */
+  gal_data_t    *fitestval;  /* Values to estimate over fit.             */
+  int             fitwhtid;  /* Code for the nature of the weight column.*/
 };
 
 #endif
