@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -190,7 +190,8 @@ sky(struct statisticsparams *p)
 
   /* Find the Sky and Sky standard deviation on the tiles. */
   if(!cp->quiet) gettimeofday(&t1, NULL);
-  gal_threads_spin_off(sky_on_thread, p, tl->tottiles, cp->numthreads);
+  gal_threads_spin_off(sky_on_thread, p, tl->tottiles, cp->numthreads,
+                       cp->minmapsize, cp->quietmmap);
   if(!cp->quiet)
     {
       num=gal_statistics_number(p->sky_t);
@@ -211,10 +212,12 @@ sky(struct statisticsparams *p)
 
 
   /* Remove outliers if requested. */
-  if(p->outliersigma!=0.0)
-    gal_tileinternal_no_outlier(p->sky_t, p->std_t, NULL, &p->cp.tl,
-                                p->outliersclip, p->outliersigma,
-                                p->checkskyname);
+  if(p->outliernumngb)
+    gal_tileinternal_no_outlier_local(p->sky_t, p->std_t, NULL, &cp->tl,
+                                      cp->interpmetric, p->outliernumngb,
+                                      cp->numthreads, p->outliersclip,
+                                      p->outliersigma, p->checkskyname,
+                                      "--outliernumngb");
 
 
   /* Interpolate the Sky and its standard deviation. */
@@ -284,7 +287,8 @@ sky(struct statisticsparams *p)
   gal_tile_full_values_write(p->std_t, tl, !p->ignoreblankintiles, outname,
                              NULL, PROGRAM_NAME);
   p->sky_t->name = p->std_t->name = NULL;
-  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
+  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1,
+                              p->cp.quiet);
   gal_fits_key_write_config(&p->cp.okeys, "Statistics configuration",
                             "STATISTICS-CONFIG", outname, "0");
   if(!cp->quiet)
