@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -257,7 +257,8 @@ ui_add_to_single_value(struct argp_option *option, char *arg,
   else
     {
       /* Read the string of numbers. */
-      inputs=gal_options_parse_list_of_numbers(arg, filename, lineno);
+      inputs=gal_options_parse_list_of_numbers(arg, filename, lineno,
+                                               GAL_TYPE_FLOAT64);
       if(inputs->size==0)
         error(EXIT_FAILURE, 0, "'--%s' needs a value", option->name);
 
@@ -323,7 +324,8 @@ ui_read_quantile_range(struct argp_option *option, char *arg,
     }
 
   /* Parse the inputs. */
-  in=gal_options_parse_list_of_numbers(arg, filename, lineno);
+  in=gal_options_parse_list_of_numbers(arg, filename, lineno,
+                                       GAL_TYPE_FLOAT64);
 
   /* Check if there was only two numbers. */
   if(in->size!=1 && in->size!=2)
@@ -851,7 +853,7 @@ ui_read_columns_in_one(struct statisticsparams *p)
 
       /* Read the different comma-separated strings into an array (within a
          'gal_data_t'). */
-      strs=gal_options_parse_csv_strings_raw(tmp->v, NULL, 0);
+      strs=gal_options_parse_csv_strings_to_data(tmp->v, NULL, 0);
       strarr=strs->array;
 
       /* Add each array element to the final list of columns. */
@@ -1183,8 +1185,8 @@ ui_preparations(struct statisticsparams *p)
                                PROGRAM_NAME);
           else
             {
-              gal_checkset_writable_remove(tl->tilecheckname, 0,
-                                           cp->dontdelete);
+              gal_checkset_writable_remove(tl->tilecheckname, p->inputname,
+                                           0, cp->dontdelete);
               gal_table_write(check, NULL, NULL, cp->tableformat,
                               tl->tilecheckname, "TABLE", 0);
             }
@@ -1206,7 +1208,7 @@ ui_preparations(struct statisticsparams *p)
       /* Only keep the elements we want. Note that if we have more than one
          column, we need to move the same rows in both (otherwise their
          widths won't be equal). */
-      if(p->input->next) gal_blank_remove_rows(p->input, NULL);
+      if(p->input->next) gal_blank_remove_rows(p->input, NULL, 0);
       else               gal_blank_remove(p->input);
 
       /* Make sure there actually are any (non-blank) elements left. */
@@ -1232,7 +1234,7 @@ ui_preparations(struct statisticsparams *p)
   p->cp.keepinputdir=keepinputdir;
 
   /* Make sure the output doesn't already exist. */
-  gal_checkset_writable_remove(p->cp.output, p->cp.keep,
+  gal_checkset_writable_remove(p->cp.output, p->inputname, p->cp.keep,
                                p->cp.dontdelete);
 
   /* Set the fit-estimate column, and prepare the weight based on the
