@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2020-2022 Free Software Foundation, Inc.
+Copyright (C) 2020-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -42,6 +42,7 @@ enum arithmetic_operators
   ARITHMETIC_TABLE_OP_DISTANCEFLAT,
   ARITHMETIC_TABLE_OP_DATETOMILLISEC,
   ARITHMETIC_TABLE_OP_DISTANCEONSPHERE,
+  ARITHMETIC_TABLE_OP_SORTEDTOINTERVAL,
 };
 
 
@@ -53,14 +54,12 @@ enum arithmetic_operators
 /* Functions */
 void
 arithmetic_init(struct tableparams *p, struct arithmetic_token **arith,
-                gal_list_str_t **toread, size_t *totcalled, char *expression);
+                gal_list_str_t **colstoread, size_t *totcalled,
+                char *expression, gal_data_t *colinfo, size_t numcols);
 
 void
 arithmetic_token_free(struct arithmetic_token *list);
 
-void
-arithmetic_indexs_final(struct tableparams *p, size_t *colmatch);
-
 void
 arithmetic_operate(struct tableparams *p);
 
