@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2020, Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -24,9 +24,11 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #define MAIN_H
 
 /* Include necessary headers */
+#include <gsl/gsl_rng.h>
 #include <gnuastro/data.h>
 
 #include <gnuastro-internal/options.h>
+#include <gnuastro-internal/arithmetic-set.h>
 
 /* Progarm names.  */
 #define PROGRAM_NAME   "Table"         /* Program full name.       */
@@ -42,6 +44,7 @@ enum select_types
  SELECT_TYPE_OUTPOLYGON,
  SELECT_TYPE_EQUAL,
  SELECT_TYPE_NOTEQUAL,
+ SELECT_TYPE_NOBLANK,
 
  /* This marks the total number of row-selection criteria. */
  SELECT_TYPE_NUMBER,
@@ -61,17 +64,24 @@ struct arithmetic_token
 {
   /* First layer elements. */
   int            operator;  /* OPERATOR: Code of operator.                */
+  int               inlib;  /* OPERATOR: if operator is in library.       */
   size_t     num_operands;  /* OPERATOR: Number of required operands.     */
   size_t            index;  /* OPERAND: Index in requested columns.       */
+  char       *id_at_usage;  /* OPERAND: col identifier at usage time.     */
+  size_t     num_at_usage;  /* OPERAND: col number at usage.              */
   gal_data_t    *constant;  /* OPERAND: a constant/single number.         */
+  gal_data_t     *loadcol;  /* OPERAND: column from another file.         */
+  char          *name_def;  /* Name given to the 'set-' operator.         */
+  char          *name_use;  /* If this a usage of a name.                 */
   struct arithmetic_token *next;  /* Pointer to next token.               */
 };
 
+/* For arithmetic operations. */
 struct column_pack
 {
   size_t                    start; /* Starting ind. in requested columns. */
-  size_t                numsimple; /* Number of simple columns.           */
-  struct arithmetic_token *tokens; /* Arithmetic tokens to use.           */
+  size_t                numsimple; /* Num. of simple columns (no change). */
+  struct arithmetic_token  *arith; /* Arithmetic tokens to use.           */
   struct column_pack        *next; /* Next output column.                 */
 };
 
@@ -90,25 +100,42 @@ struct tableparams
   gal_list_str_t     *columns;  /* List of given columns.               */
   uint8_t         information;  /* ==1: only print FITS information.    */
   uint8_t     colinfoinstdout;  /* ==1: print column metadata in CL.    */
+  uint8_t            rowfirst;  /* Do row-based operations first.       */
   gal_data_t           *range;  /* Range to limit output.               */
   gal_data_t       *inpolygon;  /* Columns to check if inside polygon.  */
   gal_data_t      *outpolygon;  /* Columns to check if outside polygon. */
   gal_data_t         *polygon;  /* Values of vertices of the polygon.   */
   gal_data_t           *equal;  /* Values to keep in output.            */
   gal_data_t        *notequal;  /* Values to not include in output.     */
+  gal_list_str_t   *noblankll;  /* Remove rows with blank values.       */
+  gal_list_str_t  *noblankend;  /* Similar to noblank but at end.       */
   char                  *sort;  /* Column name or number for sorting.   */
   uint8_t          descending;  /* Sort columns in descending order.    */
   size_t                 head;  /* Output only the no. of top rows.     */
   size_t                 tail;  /* Output only the no. of bottom rows.  */
-  gal_data_t         *noblank;  /* Remove rows that have blank.         */
-  gal_list_str_t *catcolumnfile; /* Filename to concat column wise.     */
-  gal_list_str_t *catcolumnhdu;  /* HDU/extension for the catcolumn.    */
+  gal_data_t        *rowrange;  /* Output rows in row-counter range.    */
+  size_t            rowrandom;  /* Number of rows to show randomly.     */
+  uint8_t             envseed;  /* Use the environment for random seed. */
+  gal_list_str_t *catcolumnfile;/* Filename to concat column wise.      */
+  gal_list_str_t *catcolumnhdu; /* HDU/extension for the catcolumn.     */
   gal_list_str_t  *catcolumns;  /* List of columns to concatenate.      */
   uint8_t    catcolumnrawname;  /* Don't modify name of appended col.   */
+  gal_data_t      *fromvector;  /* Extract columns from a vector column.*/
+  uint8_t         keepvectfin;  /* Keep in.s --tovector & --fromvector. */
+  gal_list_str_t    *tovector;  /* Merge columns into a vector column.  */
+  uint8_t           transpose;  /* Transpose vector columns.            */
+  gal_list_str_t  *catrowfile;  /* Filename to concat column wise.      */
+  gal_list_str_t   *catrowhdu;  /* HDU/extension for the catcolumn.     */
   gal_data_t     *colmetadata;  /* Set column metadata.                 */
+  uint8_t             txteasy;  /* Easy/simple to ready txt output.     */
+  char          *txtf32fmtstr;  /* Floating point formats (exp, flt).   */
+  char          *txtf64fmtstr;  /* Floating point formats (exp, flt).   */
+  int         txtf32precision;  /* Precision of float32 in text.        */
+  int         txtf64precision;  /* Precision of float32 in text.        */
 
   /* Internal. */
-  struct column_pack *outcols;  /* Output column packages.              */
+  struct column_pack *colpack;  /* Output column packages.              */
+  gal_data_t         *noblank;  /* Remove rows that have blank.         */
   gal_data_t           *table;  /* Linked list of output table columns. */
   struct wcsprm          *wcs;  /* WCS structure for conversion.        */
   int                    nwcs;  /* Number of WCS structures.            */
@@ -123,6 +150,12 @@ struct tableparams
   time_t              rawtime;  /* Starting time of the program.        */
   gal_data_t       **colarray;  /* Array of columns, with arithmetic.   */
   size_t          numcolarray;  /* Number of elements in 'colarray'.    */
+  gsl_rng                *rng;  /* Main random number generator.        */
+  const char        *rng_name;  /* Name of random number generator.     */
+  unsigned long int  rng_seed;  /* Random number generator seed.        */
+  size_t            *colmatch;  /* Number of matches found for columns. */
+  uint8_t        txtf32format;  /* Floating point formats (exp, flt).   */
+  uint8_t        txtf64format;  /* Floating point formats (exp, flt).   */
 
   /* For arithmetic operators. */
   gal_list_str_t  *wcstoimg_p;  /* Pointer to the node.                 */
