@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2022 Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,7 +33,9 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* Option groups particular to this program. */
 enum program_args_groups
 {
-  UI_GROUP_OUTROWS = GAL_OPTIONS_GROUP_AFTER_COMMON,
+  UI_GROUP_PRECEDENCE = GAL_OPTIONS_GROUP_AFTER_COMMON,
+  UI_GROUP_OUTCOLS,
+  UI_GROUP_OUTROWS,
 };
 
 
@@ -41,8 +43,8 @@ enum program_args_groups
 
 /* Available letters for short options:
 
-   a d f g j k l p t v x y z
-   A B G J O Q Y
+   a g j l t v x y z
+   G J Q
 */
 enum option_keys_enum
 {
@@ -67,15 +69,25 @@ enum option_keys_enum
   UI_KEY_CATROWFILE      = 'R',
   UI_KEY_CATROWHDU       = 'X',
   UI_KEY_COLMETADATA     = 'm',
+  UI_KEY_TXTEASY         = 'Y',
+  UI_KEY_TXTF32FORMAT    = 'f',
+  UI_KEY_TXTF64FORMAT    = 'A',
+  UI_KEY_TXTF32PRECISION = 'p',
+  UI_KEY_TXTF64PRECISION = 'B',
+  UI_KEY_KEEPVECTFIN     = 'k',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
   UI_KEY_POLYGON         = 1000,
   UI_KEY_ENVSEED,
-  UI_KEY_ROWLIMIT,
+  UI_KEY_ROWRANGE,
+  UI_KEY_TOVECTOR,
+  UI_KEY_ROWFIRST,
   UI_KEY_ROWRANDOM,
   UI_KEY_INPOLYGON,
+  UI_KEY_TRANSPOSE,
   UI_KEY_OUTPOLYGON,
+  UI_KEY_FROMVECTOR,
   UI_KEY_CATCOLUMNRAWNAME,
 };
 
@@ -89,6 +101,9 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct tableparams *p);
 void
 ui_list_select_free(struct list_select *list, int freevalue);
 
+void
+ui_colpack_free(struct column_pack *list);
+
 void
 ui_free_report(struct tableparams *p);
 
