@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2021, Free Software Foundation, Inc.
+Copyright (C) 2017-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -107,7 +107,22 @@ gal_blank_alloc_write(uint8_t type)
 void
 gal_blank_initialize(gal_data_t *input)
 {
-  GAL_TILE_PARSE_OPERATE(input, NULL, 0, 0, {*i=b;});
+  size_t i;
+  char **strarr;
+
+  /* For strings, we will initialize the full array, for numerical data
+     types we will consider tiles. */
+  if(input->type==GAL_TYPE_STRING)
+    {
+      strarr=input->array;
+      for(i=0;i<input->size;++i)
+        {
+          if(strarr[i]) free(strarr[i]);
+          gal_checkset_allocate_copy(GAL_BLANK_STRING, &strarr[i]);
+        }
+    }
+  else
+    {GAL_TILE_PARSE_OPERATE(input, NULL, 0, 0, {*i=b;});}
 }
 
 
@@ -317,6 +332,11 @@ gal_blank_as_string(uint8_t type, int width)
 int
 gal_blank_is(void *pointer, uint8_t type)
 {
+  /* A small sanity check. */
+  if(pointer==NULL)
+    error(EXIT_FAILURE, 0, "%s: input pointer is NULL", __func__);
+
+  /* Do the checks based on the type. */
   switch(type)
     {
     /* Numeric types */
@@ -332,7 +352,8 @@ gal_blank_is(void *pointer, uint8_t type)
     case GAL_TYPE_FLOAT64:   return isnan( *(double *)(pointer) );
 
     /* String. */
-    case GAL_TYPE_STRING:    if(!strcmp(pointer,GAL_BLANK_STRING)) return 1;
+    case GAL_TYPE_STRING:
+      if(!strcmp(pointer,GAL_BLANK_STRING)) return 1; else return 0;
 
     /* Complex types */
     case GAL_TYPE_COMPLEX32:
@@ -442,10 +463,12 @@ gal_blank_present(gal_data_t *input, int updateflag)
     /* String. */
     case GAL_TYPE_STRING:
       if(input!=block)
-        error(EXIT_FAILURE, 0, "%s: tile mode is currently not supported for "
-              "strings", __func__);
+        error(EXIT_FAILURE, 0, "%s: tile mode is currently not "
+              "supported for strings", __func__);
       strf = (str=input->array) + input->size;
-      do if(!strcmp(*str,GAL_BLANK_STRING)) return 1; while(++str<strf);
+      do
+        if(*str==NULL || !strcmp(*str,GAL_BLANK_STRING)) return 1;
+      while(++str<strf);
       break;
 
     /* Complex types */
@@ -486,17 +509,34 @@ gal_blank_present(gal_data_t *input, int updateflag)
 size_t
 gal_blank_number(gal_data_t *input, int updateflag)
 {
+  size_t nblank;
+  char **strarr;
   gal_data_t *number;
-  size_t num_not_blank;
+  size_t i, num_not_blank;
 
   if(input)
     {
       if( gal_blank_present(input, updateflag) )
         {
-          number=gal_statistics_number(input);
-          num_not_blank=((size_t *)(number->array))[0];
-          gal_data_free(number);
-          return input->size - num_not_blank;
+          if(input->type==GAL_TYPE_STRING)
+            {
+              nblank=0;
+              strarr=input->array;
+              for(i=0;i<input->size;++i)
+                {
+                  if( strarr[i]==NULL
+                      || strcmp(strarr[i], GAL_BLANK_STRING)==0 )
+                    ++nblank;
+                }
+              return nblank;
+            }
+          else
+            {
+              number=gal_statistics_number(input);
+              num_not_blank=((size_t *)(number->array))[0];
+              gal_data_free(number);
+              return input->size - num_not_blank;
+            }
         }
       else
         return 0;
@@ -517,25 +557,37 @@ gal_blank_number(gal_data_t *input, int updateflag)
 #define FLAG_BLANK(IT) {                                                \
     IT b, *a=input->array;                                              \
     gal_blank_write(&b, input->type);                                   \
-    if(b==b) /* Blank value can be checked with the equal comparison */ \
-      do { *o = *a==b;  ++a; } while(++o<of);                           \
-    else     /* Blank value will fail with the equal comparison */      \
-      do { *o = *a!=*a; ++a; } while(++o<of);                           \
+    if(b==b) /* Blank value can be checked with the equal. */           \
+      do {*o = blank1_not0 ? *a==b  : *a!=b;  ++a;} while(++o<of);      \
+    else     /* Blank value will fail with the equal comparison. */     \
+      do {*o = blank1_not0 ? *a!=*a : *a==*a; ++a;} while(++o<of);      \
   }
-gal_data_t *
-gal_blank_flag(gal_data_t *input)
+static gal_data_t *
+blank_flag(gal_data_t *input, int blank1_not0)
 {
   uint8_t *o, *of;
   gal_data_t *out;
   char **str=input->array, **strf=str+input->size;
 
+  /* The datasets may be empty. In this case, the output should also be
+     empty, but with the standard 'uint8' type of a flag (we can have
+     tables and images with 0 rows or pixels!). */
+  if(input->size==0 || input->array==NULL)
+    {
+      out=gal_data_alloc_empty(input->ndim, input->minmapsize,
+                               input->quietmmap);
+      out->type=GAL_TYPE_UINT8;
+      return out;
+    }
+
+  /* Do all the checks and allocations if a blank is actually present! */
   if( gal_blank_present(input, 0) )
     {
       /* Allocate a non-cleared output array, we are going to parse the
          input and fill in each element. */
       out=gal_data_alloc(NULL, GAL_TYPE_UINT8, input->ndim, input->dsize,
-                         input->wcs, 0, input->minmapsize, input->quietmmap,
-                         NULL, "bool", NULL);
+                         input->wcs, 0, input->minmapsize,
+                         input->quietmmap, NULL, "bool", NULL);
 
       /* Set the pointers for easy looping. */
       of=(o=out->array)+input->size;
@@ -587,6 +639,26 @@ gal_blank_flag(gal_data_t *input)
 
 
 
+gal_data_t *
+gal_blank_flag(gal_data_t *input)
+{
+  return blank_flag(input, 1);
+}
+
+
+
+
+
+gal_data_t *
+gal_blank_flag_not(gal_data_t *input)
+{
+  return blank_flag(input, 0);
+}
+
+
+
+
+
 /* Write a blank value in the input anywhere that the flag dataset is not
    zero or not blank. */
 #define BLANK_FLAG_APPLY(IT) {                                          \
@@ -688,6 +760,11 @@ gal_blank_flag_remove(gal_data_t *input, gal_data_t *flag)
     error(EXIT_FAILURE, 0, "%s: the 'flag' argument doesn't have the same "
           "size as the 'input' argument", __func__);
 
+  /* If there is no elements in the input or the flag (we have already
+     confirmed that they are the same size), then just return (nothing is
+     necessary to do). */
+  if(flag->size==0 || flag->array==NULL) return;
+
   /* Shift all non-blank elements to the start of the array. */
   switch(input->type)
     {
@@ -705,9 +782,9 @@ gal_blank_flag_remove(gal_data_t *input, gal_data_t *flag)
       strarr=input->array;
       for(i=0;i<input->size;++i)
         {
-          if( *f && *f!=GAL_BLANK_UINT8 )        /* Flagged to be removed */
+          if( *f && *f!=GAL_BLANK_UINT8 )    /* Flagged to be removed */
             { free(strarr[i]); strarr[i]=NULL; }
-          else strarr[num++]=strarr[i];          /* Keep. */
+          else strarr[num++]=strarr[i];      /* Keep. */
           ++f;
         }
       break;
@@ -725,6 +802,90 @@ gal_blank_flag_remove(gal_data_t *input, gal_data_t *flag)
 
 
 
+/* Remove flagged elements from a dataset (which may not necessarily
+   blank), convert it to a 1D dataset and adjust the size properly. In
+   practice this function doesn't 'realloc' the input array, all it does is
+   to shift the blank eleemnts to the end and adjust the size elements of
+   the 'gal_data_t'. */
+#define BLANK_FLAG_REMOVE_D0(IT) {                                      \
+    IT *a=input->array, *af=a+input->size, *o=input->array;             \
+    do {                                                                \
+      printf("\n%s: HERE\n", __func__);                                   \
+      if(*f) a+=nelem;                                                  \
+      else {num++; for(i=0;i<nelem;++i) {*o++=*a; ++a; printf("%zu ", i);} } \
+      ++f;                                                              \
+    }                                                                   \
+    while(a<af);                                                        \
+  }
+static void
+blank_flag_remove_dim0(gal_data_t *input, gal_data_t *flag)
+{
+  size_t i, num=0;
+  uint8_t it=input->type;
+  uint8_t *f=flag->array;
+  size_t nelem = input->ndim==1 ? 1 : input->size/input->dsize[0];
+
+  /* Sanity check. */
+  if(flag->type!=GAL_TYPE_UINT8)
+    error(EXIT_FAILURE, 0, "%s: the 'flag' argument has a '%s' type, it "
+          "must have an unsigned 8-bit type", __func__,
+          gal_type_name(flag->type, 1));
+  if(input->dsize[0]!=flag->dsize[0])
+    error(EXIT_FAILURE, 0, "%s: the 'flag' argument doesn't have the same "
+          "size as the 'input' argument", __func__);
+
+  /* This is a special function! */
+  if(flag->ndim!=1)
+    error(EXIT_FAILURE, 0, "%s: this function's 'flag' should only "
+          "have a single dimension", __func__);
+  if(input->ndim==flag->ndim)
+    error(EXIT_FAILURE, 0, "%s: this function is only for scenarios "
+          "where 'flag' is single-dimensional and the input is "
+          "multi-dimensional. Please use 'gal_blank_flag_remove' when "
+          "the input and flag have the same number of dimensions",
+          __func__);
+
+  /* If there is no elements in the input or the flag (we have already
+     confirmed that they are the same size), then just return (nothing is
+     necessary to do). */
+  if(flag->size==0 || flag->array==NULL) return;
+
+  /* Move all the non-flagged elements to the start of the array. */
+  for(i=0;i<flag->size;++i)
+    if(f[i]==0)
+      memmove(gal_pointer_increment(input->array, (num++)*nelem, it),
+              gal_pointer_increment(input->array, i*nelem,       it),
+              nelem*gal_type_sizeof(it));
+
+  /* Shift all non-blank elements to the start of the array.
+  switch(input->type)
+    {
+    case GAL_TYPE_UINT8:    BLANK_FLAG_REMOVE_D0( uint8_t  );    break;
+    case GAL_TYPE_INT8:     BLANK_FLAG_REMOVE_D0( int8_t   );    break;
+    case GAL_TYPE_UINT16:   BLANK_FLAG_REMOVE_D0( uint16_t );    break;
+    case GAL_TYPE_INT16:    BLANK_FLAG_REMOVE_D0( int16_t  );    break;
+    case GAL_TYPE_UINT32:   BLANK_FLAG_REMOVE_D0( uint32_t );    break;
+    case GAL_TYPE_INT32:    BLANK_FLAG_REMOVE_D0( int32_t  );    break;
+    case GAL_TYPE_UINT64:   BLANK_FLAG_REMOVE_D0( uint64_t );    break;
+    case GAL_TYPE_INT64:    BLANK_FLAG_REMOVE_D0( int64_t  );    break;
+    case GAL_TYPE_FLOAT32:  BLANK_FLAG_REMOVE_D0( float    );    break;
+    case GAL_TYPE_FLOAT64:  BLANK_FLAG_REMOVE_D0( double   );    break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: type code %d not recognized",
+            __func__, input->type);
+    }
+  */
+
+  /* Adjust the size elements of the dataset. */
+  input->size=1;
+  input->dsize[0]=num;
+  for(i=0;i<input->ndim;++i) input->size*=input->dsize[i];
+}
+
+
+
+
+
 /* Remove blank elements from a dataset, convert it to a 1D dataset and
    adjust the size properly. In practice this function doesn't 'realloc'
    the input array, all it does is to shift the blank eleemnts to the end
@@ -745,8 +906,8 @@ gal_blank_remove(gal_data_t *input)
 
   /* This function currently assumes a contiguous patch of memory. */
   if(input->block && input->ndim!=1 )
-    error(EXIT_FAILURE, 0, "%s: tiles in datasets with more dimensions than "
-          "1 are not yet supported. Your input has %zu dimensions",
+    error(EXIT_FAILURE, 0, "%s: tiles in datasets with more dimensions "
+          "than 1 are not yet supported. Your input has %zu dimensions",
           __func__, input->ndim);
 
   /* If the dataset doesn't have blank values, then just get the size. */
@@ -784,7 +945,7 @@ gal_blank_remove(gal_data_t *input)
   input->dsize[0]=input->size=num;
 
   /* Set the flags to mark that there is no blanks. */
-  input->flag |= GAL_DATA_FLAG_BLANK_CH;
+  input->flag |=  GAL_DATA_FLAG_BLANK_CH;
   input->flag &= ~GAL_DATA_FLAG_HASBLANK;
 }
 
@@ -812,11 +973,23 @@ gal_blank_remove_realloc(gal_data_t *input)
 
 
 static gal_data_t *
-blank_remove_in_list_merge_flags(gal_data_t *thisdata, gal_data_t *flag)
+blank_remove_in_list_merge_flags(gal_data_t *thisdata, gal_data_t *flag,
+                                 int onlydim0)
 {
   size_t i;
   uint8_t *u, *tu;
   gal_data_t *flagtmp;
+  static int warningprinted=0;
+
+  /* Ignore the dataset if it has more than one dimension and 'onlydim0' is
+     called*/
+  if(onlydim0 && thisdata->ndim>1 && warningprinted==0)
+    {
+      warningprinted=1;
+      error(EXIT_SUCCESS, 0, "%s: WARNING: multi-dimensional columns "
+            "are not supported when 'onlydim0' is non-zero", __func__);
+      return NULL;
+    }
 
   /* Build the flag of blank elements for this column. */
   flagtmp=gal_blank_flag(thisdata);
@@ -849,7 +1022,8 @@ blank_remove_in_list_merge_flags(gal_data_t *thisdata, gal_data_t *flag)
 
 /* Remove any row that has a blank in any of the given columns. */
 gal_data_t *
-gal_blank_remove_rows(gal_data_t *columns, gal_list_sizet_t *column_indexs)
+gal_blank_remove_rows(gal_data_t *columns, gal_list_sizet_t *column_indexs,
+                      int onlydim0)
 {
   size_t i;
   gal_list_sizet_t *tcol;
@@ -868,20 +1042,23 @@ gal_blank_remove_rows(gal_data_t *columns, gal_list_sizet_t *column_indexs)
         /* If the given index is larger than the number of elements in the
            input list, then print an error. */
         if(tmp==NULL)
-          error(EXIT_FAILURE, 0, "%s: input list has %zu elements, but the "
-                "column %zu (counting from zero) has been requested",
+          error(EXIT_FAILURE, 0, "%s: input list has %zu elements, but "
+                "the column %zu (counting from zero) has been requested",
                 __func__, gal_list_data_number(columns), tcol->v);
 
         /* Build the flag of blank elements for this column. */
-        flag=blank_remove_in_list_merge_flags(tmp, flag);
+        flag=blank_remove_in_list_merge_flags(tmp, flag, onlydim0);
       }
   else
     for(tmp=columns; tmp!=NULL; tmp=tmp->next)
-      flag=blank_remove_in_list_merge_flags(tmp, flag);
+      flag=blank_remove_in_list_merge_flags(tmp, flag, onlydim0);
 
   /* Now that the flags have been set, remove the rows. */
   for(tmp=columns; tmp!=NULL; tmp=tmp->next)
-    gal_blank_flag_remove(tmp, flag);
+    {
+      if(tmp->ndim==1 || onlydim0==0) gal_blank_flag_remove(tmp, flag);
+      else blank_flag_remove_dim0(tmp, flag);
+    }
 
   /* For a check.
   double *d1=columns->array, *d2=columns->next->array;
