@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2022 Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -123,6 +123,7 @@ gal_data_initialize(gal_data_t *data, void *array, uint8_t type,
      the default values are used if/when printing.*/
   data->flag       = 0;
   data->status     = 0;
+  data->disp_width = -1;
   data->next       = NULL;
   data->ndim       = ndim;
   data->type       = type;
@@ -130,10 +131,11 @@ gal_data_initialize(gal_data_t *data, void *array, uint8_t type,
   data->mmapname   = NULL;
   data->quietmmap  = quietmmap;
   data->minmapsize = minmapsize;
+  data->disp_precision=GAL_BLANK_INT;
+  data->disp_fmt=GAL_TABLE_DISPLAY_FMT_INVALID;
   gal_checkset_allocate_copy(name, &data->name);
   gal_checkset_allocate_copy(unit, &data->unit);
   gal_checkset_allocate_copy(comment, &data->comment);
-  data->disp_fmt=data->disp_width=data->disp_precision=-1;
 
 
   /* Copy the WCS structure. */
@@ -211,6 +213,37 @@ gal_data_initialize(gal_data_t *data, void *array, uint8_t type,
 
 
 
+/* Allocate an empty (meta) dataset with a certain number of dimensions,
+   but no 'array' component, and all 'size' elements set to zero. */
+gal_data_t *
+gal_data_alloc_empty(size_t ndim, size_t minmapsize, int quietmmap)
+{
+  gal_data_t *out;
+  size_t i, *dsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim,
+                                        0, __func__, "dsize");
+
+  /* Fill the 'dsize' array with 1 values (so too much space isn't
+     allocated!), then allocate it. */
+  for(i=0;i<ndim;++i) dsize[i]=1;
+  out=gal_data_alloc(NULL, GAL_TYPE_UINT8, ndim, dsize, NULL, 0,
+                     minmapsize, quietmmap, NULL, NULL, NULL);
+
+  /* Update the sizes. */
+  out->size=0;
+  for(i=0;i<ndim;++i) out->dsize[i]=0;
+
+  /* Clean up the allocated space for 'out->array', and the extra 'dsize',
+     then return. */
+  free(out->array);
+  out->array=NULL;
+  free(dsize);
+  return out;
+}
+
+
+
+
+
 /* Free the allocated contents of a data structure, not the structure
    itsself. The reason that this function is separate from 'gal_data_free'
    is that the data structure might be allocated as an array (statically
@@ -536,7 +569,7 @@ data_copy_from_string(gal_data_t *from, gal_data_t *to)
       }                                                                 \
   }
 
-#define COPY_TO_STR_FLT(CTYPE, BLANK) {                                 \
+#define COPY_TO_STR_FLT(CTYPE, BLANK, FMT) {                            \
     CTYPE *a=from->array;                                               \
     for(i=0;i<from->size;++i)                                           \
       {                                                                 \
@@ -544,7 +577,7 @@ data_copy_from_string(gal_data_t *from, gal_data_t *to)
         else             isblank = a[i]==BLANK ? 1 : 0;                 \
         if(isblank==0)                                                  \
           {                                                             \
-            if( asprintf(&strarr[i], "%f", a[i])<0 )                    \
+            if( asprintf(&strarr[i], FMT, a[i])<0 )                    \
               error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__); \
           }                                                             \
         else gal_checkset_allocate_copy(GAL_BLANK_STRING, &strarr[i]);  \
@@ -557,7 +590,8 @@ static void
 data_copy_to_string(gal_data_t *from, gal_data_t *to)
 {
   size_t i;
-  int isblank;
+  char *fltfmt;
+  int isblank, leftadjust=1;
   char **strarr=to->array, **instrarr=from->array;
 
   /* Sanity check */
@@ -568,6 +602,28 @@ data_copy_to_string(gal_data_t *from, gal_data_t *to)
           "('block' element must be NULL). Please contact us at %s so we "
           "can implement this feature", __func__, PACKAGE_BUGREPORT);
 
+  /* For the two floating point formats, the user may have given a certain
+     width and precision. */
+  if(from->disp_width>0 || from->disp_precision>0)
+    {
+      if(from->disp_width>0)    /* Both width and precision are given. */
+        {
+          if( asprintf(&fltfmt, "%%%s%d.%df",
+                       leftadjust ? "-" : "", from->disp_width,
+                       from->disp_precision)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation error "
+                  "(with width)", __func__);
+        }
+      else                      /* Only precision is given. */
+        {
+          if( asprintf(&fltfmt, "%%.%df", from->disp_precision)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation error "
+                  "(without width)", __func__);
+        }
+    }
+  else
+    gal_checkset_allocate_copy("%g", &fltfmt);
+
   /* Do the copying */
   switch(from->type)
     {
@@ -596,10 +652,10 @@ data_copy_to_string(gal_data_t *from, gal_data_t *to)
       COPY_TO_STR_INT(int64_t,  GAL_BLANK_INT64,  "%"PRId64); break;
 
     case GAL_TYPE_FLOAT32:
-      COPY_TO_STR_FLT(float, GAL_BLANK_FLOAT32);              break;
+      COPY_TO_STR_FLT(float, GAL_BLANK_FLOAT32, fltfmt);      break;
 
     case GAL_TYPE_FLOAT64:
-      COPY_TO_STR_FLT(double, GAL_BLANK_FLOAT32);             break;
+      COPY_TO_STR_FLT(double, GAL_BLANK_FLOAT64, fltfmt);     break;
 
     case GAL_TYPE_STRING:
       for(i=0;i<from->size;++i)
@@ -868,9 +924,22 @@ gal_data_copy_to_allocated(gal_data_t *in, gal_data_t *out)
   /* Correct the sizes of the output to be the same as the input. If it is
      equal, there is no problem, if not, the size information will be
      changed, so if you want to use this allocated space again, be sure to
-     re-set the size parameters. */
+     re-set the size parameters.
+
+     As defined in 'gal_data_initialize', when there is no dataset, 'dsize'
+     should be NULL. So if 'in->dsize==NULL', but 'out->dsize!=NULL', then
+     we should free 'out->dsize' and set it to NULL. Alternatively, if
+     'out' hasn't allocated a 'dsize', but 'in' already has one, allocate
+     'out->dsize', then fill it. */
   out->size=in->size;
-  memcpy(out->dsize, in->dsize, in->ndim * sizeof *(in->dsize) );
+  if(in->dsize)
+    {
+      if(out->dsize==NULL)
+        out->dsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, in->ndim, 0,
+                                        __func__, "out->dsize");
+      memcpy(out->dsize, in->dsize, in->ndim * sizeof *(in->dsize) );
+    }
+  else if(out->dsize) { free(out->dsize); out->dsize=NULL; }
 }
 
 
