@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2021, Free Software Foundation, Inc.
+Copyright (C) 2017-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -71,6 +71,15 @@ __BEGIN_C_DECLS  /* From C++ preparations */
                                      | GAL_ARITHMETIC_FLAG_FREE    \
                                      | GAL_ARITHMETIC_FLAG_NUMOK )
 
+/* Operator fixed strings. */
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_PREFIX     "load-col-"
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_HDU        "-hdu-"
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_FILE       "-from-"
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_PREFIX_LEN strlen(GAL_ARITHMETIC_OPSTR_LOADCOL_PREFIX)
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_HDU_LEN    strlen(GAL_ARITHMETIC_OPSTR_LOADCOL_HDU)
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_FILE_LEN   strlen(GAL_ARITHMETIC_OPSTR_LOADCOL_FILE)
+
+
 
 /* Identifiers for each operator. */
 enum gal_arithmetic_operators
@@ -93,6 +102,7 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_OR,           /*   ||    */
   GAL_ARITHMETIC_OP_NOT,          /*   !     */
   GAL_ARITHMETIC_OP_ISBLANK,      /* Similar to isnan() for floats. */
+  GAL_ARITHMETIC_OP_ISNOTBLANK,   /* Inverse of 'isblank'. */
   GAL_ARITHMETIC_OP_WHERE,        /*   ?:    */
 
   GAL_ARITHMETIC_OP_BITAND,       /*   &     */
@@ -114,7 +124,7 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_ASIN,         /* Inverse sine (output in deg). */
   GAL_ARITHMETIC_OP_ACOS,         /* Inverse cosine (output in deg). */
   GAL_ARITHMETIC_OP_ATAN,         /* Inverse tangent (output in deg). */
-  GAL_ARITHMETIC_OP_ATAN2,        /* Inv. atan (preserves quad, out in deg). */
+  GAL_ARITHMETIC_OP_ATAN2,        /* Inv. atan (preserves quad, in deg). */
   GAL_ARITHMETIC_OP_SINH,         /* Hyperbolic sine. */
   GAL_ARITHMETIC_OP_COSH,         /* Hyperbolic cosine. */
   GAL_ARITHMETIC_OP_TANH,         /* Hyperbolic tangent. */
@@ -122,13 +132,38 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_ACOSH,        /* Inverse hyperbolic cosine. */
   GAL_ARITHMETIC_OP_ATANH,        /* Inverse hyperbolic tangent. */
 
+  GAL_ARITHMETIC_OP_E,            /* The base of natural logirithm. */
+  GAL_ARITHMETIC_OP_PI,           /* Circle circumference by diameter. */
+  GAL_ARITHMETIC_OP_C,            /* The speed of light. */
+  GAL_ARITHMETIC_OP_G,            /* The gravitational constant. */
+  GAL_ARITHMETIC_OP_H,            /* Plank's constant. */
+  GAL_ARITHMETIC_OP_AU,           /* Astronomical Unit (in meters). */
+  GAL_ARITHMETIC_OP_LY,           /* Light years (in meters). */
+  GAL_ARITHMETIC_OP_AVOGADRO,     /* Avogadro's number. */
+  GAL_ARITHMETIC_OP_FINESTRUCTURE,/* Fine-structure constant. */
+
   GAL_ARITHMETIC_OP_RA_TO_DEGREE, /* right ascension to decimal. */
   GAL_ARITHMETIC_OP_DEC_TO_DEGREE,/* declination to decimal. */
   GAL_ARITHMETIC_OP_DEGREE_TO_RA, /* right ascension to decimal. */
   GAL_ARITHMETIC_OP_DEGREE_TO_DEC,/* declination to decimal. */
   GAL_ARITHMETIC_OP_COUNTS_TO_MAG,/* Counts to magnitude. */
   GAL_ARITHMETIC_OP_MAG_TO_COUNTS,/* Magnitude to counts. */
+  GAL_ARITHMETIC_OP_MAG_TO_SB,    /* Magnitude to Surface Brightness. */
+  GAL_ARITHMETIC_OP_SB_TO_MAG,    /* Surface Brightness to Magnitude. */
+  GAL_ARITHMETIC_OP_COUNTS_TO_SB, /* Counts to Surface Brightness. */
+  GAL_ARITHMETIC_OP_SB_TO_COUNTS, /* Surface Brightness to counts. */
   GAL_ARITHMETIC_OP_COUNTS_TO_JY, /* Counts to Janskys with AB-mag zeropoint. */
+  GAL_ARITHMETIC_OP_JY_TO_COUNTS, /* Janskys to Counts with AB-mag zeropoint. */
+  GAL_ARITHMETIC_OP_MAG_TO_JY,    /* Magnitude to Janskys. */
+  GAL_ARITHMETIC_OP_JY_TO_MAG,    /* Janskys to Magnitude. */
+  GAL_ARITHMETIC_OP_COUNTS_TO_NANOMAGGY,/* Counts to SDSS nanomaggies. */
+  GAL_ARITHMETIC_OP_NANOMAGGY_TO_COUNTS,/* SDSS nanomaggies to counts. */
+  GAL_ARITHMETIC_OP_AU_TO_PC,     /* Astronomical units (AU) to Parsecs (PC). */
+  GAL_ARITHMETIC_OP_PC_TO_AU,     /* Parsecs (PC) to Astronomical units (AU). */
+  GAL_ARITHMETIC_OP_LY_TO_PC,     /* Astronomical units (AU) to Parsecs (PC). */
+  GAL_ARITHMETIC_OP_PC_TO_LY,     /* Parsecs (PC) to Astronomical units (AU). */
+  GAL_ARITHMETIC_OP_LY_TO_AU,     /* Light-years to Astronomical units (AU). */
+  GAL_ARITHMETIC_OP_AU_TO_LY,     /* Astronomical units (AU) to Light-years. */
 
   GAL_ARITHMETIC_OP_MINVAL,       /* Minimum value of array.               */
   GAL_ARITHMETIC_OP_MAXVAL,       /* Maximum value of array.               */
@@ -137,6 +172,8 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_MEANVAL,      /* Mean value of array.                  */
   GAL_ARITHMETIC_OP_STDVAL,       /* Standard deviation value of array.    */
   GAL_ARITHMETIC_OP_MEDIANVAL,    /* Median value of array.                */
+  GAL_ARITHMETIC_OP_UNIQUE,       /* Only return unique elements.          */
+  GAL_ARITHMETIC_OP_NOBLANK,      /* Only keep non-blank elements.         */
 
   GAL_ARITHMETIC_OP_MIN,          /* Minimum per pixel of multiple arrays. */
   GAL_ARITHMETIC_OP_MAX,          /* Maximum per pixel of multiple arrays. */
@@ -154,8 +191,10 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_MKNOISE_SIGMA,/* Fixed-sigma noise to every element.   */
   GAL_ARITHMETIC_OP_MKNOISE_POISSON,/* Poission noise on every element.    */
   GAL_ARITHMETIC_OP_MKNOISE_UNIFORM,/* Uniform noise on every element.     */
+  GAL_ARITHMETIC_OP_RANDOM_FROM_HIST,/* Randoms from a histogram (uniform).*/
+  GAL_ARITHMETIC_OP_RANDOM_FROM_HIST_RAW,/* Randoms from a histogram (raw).*/
 
-  GAL_ARITHMETIC_OP_SIZE,         /* Size of the dataset along an axis     */
+  GAL_ARITHMETIC_OP_STITCH,       /* Stitch multiple datasets together.    */
 
   GAL_ARITHMETIC_OP_TO_UINT8,     /* Convert to uint8_t.                   */
   GAL_ARITHMETIC_OP_TO_INT8,      /* Convert to int8_t.                    */
@@ -169,8 +208,17 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_TO_FLOAT64,   /* Convert to float64.                   */
 
   GAL_ARITHMETIC_OP_BOX_AROUND_ELLIPSE, /* Width/Height of box over ellipse*/
+  GAL_ARITHMETIC_OP_BOX_VERTICES_ON_SPHERE, /* Vert. from center and width*/
 
+  /* Meta operators */
   GAL_ARITHMETIC_OP_MAKENEW,      /* Build a new dataset, containing zeros.*/
+  GAL_ARITHMETIC_OP_CONSTANT,     /* Make a row with given constant. */
+  GAL_ARITHMETIC_OP_SIZE,         /* Size of the dataset along an axis     */
+  GAL_ARITHMETIC_OP_INDEX,        /* New with the index (counting from 0). */
+  GAL_ARITHMETIC_OP_COUNTER,      /* New with the index (counting from 0). */
+  GAL_ARITHMETIC_OP_INDEXONLY,    /* New with the index (counting from 0). */
+  GAL_ARITHMETIC_OP_COUNTERONLY,  /* New with the index (counting from 1). */
+  GAL_ARITHMETIC_OP_SWAP,         /* Swap the top two operands.            */
 
   GAL_ARITHMETIC_OP_LAST_CODE,    /* Last code of the library operands.    */
 };
@@ -181,6 +229,10 @@ gal_arithmetic_operator_string(int operator);
 int
 gal_arithmetic_set_operator(char *string, size_t *num_operands);
 
+gal_data_t *
+gal_arithmetic_load_col(char *str, int searchin, int ignorecase,
+                        size_t minmapsize, int quietmmap);
+
 gal_data_t *
 gal_arithmetic(int operator, size_t numthreads, int flags, ...);
 
