@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2019, Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -28,12 +28,12 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gsl/gsl_complex.h>
 
-/* When we are within Gnuastro's building process, `IN_GNUASTRO_BUILD' is
+/* When we are within Gnuastro's building process, 'IN_GNUASTRO_BUILD' is
    defined. In the build process, installation information (in particular
-   `GAL_CONFIG_SIZEOF_SIZE_T' that we need below) is kept in
-   `config.h'. When building a user's programs, this information is kept in
-   `gnuastro/config.h'. Note that all `.c' files in Gnuastro's source must
-   start with the inclusion of `config.h' and that `gnuastro/config.h' is
+   'GAL_CONFIG_SIZEOF_SIZE_T' that we need below) is kept in
+   'config.h'. When building a user's programs, this information is kept in
+   'gnuastro/config.h'. Note that all '.c' files in Gnuastro's source must
+   start with the inclusion of 'config.h' and that 'gnuastro/config.h' is
    only created at installation time (not present during the building of
    Gnuastro). */
 #ifndef IN_GNUASTRO_BUILD
@@ -71,15 +71,21 @@ enum gal_types
 {
   GAL_TYPE_INVALID,         /* Invalid (=0 by C standard).             */
 
-  GAL_TYPE_BIT,             /* 1 bit                                   */
-  GAL_TYPE_UINT8,           /* 8-bit  unsigned integer.                */
+  /* Integer types: the ordering here is used to find the output type of
+     binary operations in 'gal_type_out'. Therefore, as in the automatic C
+     type conversion, the unsigned types should be placed after (so their
+     type is preferred over a similar-width integer that is signed). */
   GAL_TYPE_INT8,            /* 8-bit  signed   integer.                */
-  GAL_TYPE_UINT16,          /* 16-bit unsigned integer.                */
+  GAL_TYPE_UINT8,           /* 8-bit  unsigned integer.                */
   GAL_TYPE_INT16,           /* 16-bit signed   integer.                */
-  GAL_TYPE_UINT32,          /* 32-bit unsigned integer.                */
+  GAL_TYPE_UINT16,          /* 16-bit unsigned integer.                */
   GAL_TYPE_INT32,           /* 32-bit signed   integer.                */
-  GAL_TYPE_UINT64,          /* 64-bit unsigned integer.                */
+  GAL_TYPE_UINT32,          /* 32-bit unsigned integer.                */
   GAL_TYPE_INT64,           /* 64-bit signed   integer.                */
+  GAL_TYPE_UINT64,          /* 64-bit unsigned integer.                */
+
+  /* Other types. */
+  GAL_TYPE_BIT,             /* 1 bit                                   */
   GAL_TYPE_FLOAT32,         /* 32-bit single precision floating point. */
   GAL_TYPE_FLOAT64,         /* 64-bit double precision floating point. */
   GAL_TYPE_COMPLEX32,       /* Complex 32-bit floating point.          */
@@ -90,13 +96,13 @@ enum gal_types
 
 
 
-/* Define system specific types. For example `size_t' is 4 and 8 bytes on
+/* Define system specific types. For example 'size_t' is 4 and 8 bytes on
    32 and 64 bit systems respectively. In both cases, the standard defines
-   `size_t' to be unsigned. A similar case exists for `long', but it is
-   signed. During `./configure' the sizeof `size_t' and `long' were found
+   'size_t' to be unsigned. A similar case exists for 'long', but it is
+   signed. During './configure' the sizeof 'size_t' and 'long' were found
    and are used to define an alias for these system specific types.
 
-   Note: we are not using `else'. This is done because by any chance, if
+   Note: we are not using 'else'. This is done because by any chance, if
    the length of these types is not what is expected (4 or 8), then the
    aliases are not defined and the compiler will crash. */
 #if GAL_CONFIG_SIZEOF_SIZE_T == 4
@@ -113,6 +119,14 @@ enum gal_types
 #define GAL_TYPE_ULONG GAL_TYPE_UINT64
 #endif
 
+#if GAL_CONFIG_SIZEOF_INT == 2
+#define GAL_TYPE_INT  GAL_TYPE_INT16
+#define GAL_TYPE_UINT GAL_TYPE_UINT16
+#elif GAL_CONFIG_SIZEOF_INT == 4
+#define GAL_TYPE_INT  GAL_TYPE_INT32
+#define GAL_TYPE_UINT GAL_TYPE_UINT32
+#endif
+
 
 
 
