@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2020, Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -37,6 +37,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro-internal/timing.h>
 #include <gnuastro-internal/checkset.h>
+#include <gnuastro-internal/tableintern.h>
 
 
 
@@ -238,7 +239,7 @@ gal_tableintern_col_print_info(gal_data_t *col, int tableformat,
 {
   size_t j;
   char **strarr;
-  int maxstrlen, width=0, precision=0;
+  int maxstrlen, width=0, precision=GAL_BLANK_INT;
 
 
   /* First do a sanity check, so we can safly stop checking in the steps
@@ -317,8 +318,18 @@ gal_tableintern_col_print_info(gal_data_t *col, int tableformat,
         }
       else width=( col->disp_width<=0 ? GAL_TABLE_DEF_WIDTH_INT
                     : col->disp_width );
-      precision=( col->disp_precision<=0 ? GAL_TABLE_DEF_PRECISION_INT
-                  : col->disp_precision );
+
+      /* For integers, there shouldn't be any default precision. If the
+         caller didn't specify it, it should just be the full set of
+         available digits. because for 'printf' the precision of integers
+         means (according to the GNU C Library manual): "the minimum number
+         of digits to appear; leading zeros are produced if necessary. If
+         you don’t specify a precision, the number is printed with as many
+         digits as it needs.  If you convert a value of zero with an
+         explicit precision of zero, then no characters at all are
+         produced". The main problem is that integer columns can contain
+         zero and in that case '%.0u' or '%.0d' will not print anything! */
+      precision=col->disp_precision;
       break;
 
 
@@ -330,8 +341,7 @@ gal_tableintern_col_print_info(gal_data_t *col, int tableformat,
       fmt[0] = tableformat==GAL_TABLE_FORMAT_TXT ? 'd' : 'I';
       width = ( col->disp_width<=0 ? GAL_TABLE_DEF_WIDTH_INT
                 : col->disp_width );
-      precision = ( col->disp_precision<=0 ? GAL_TABLE_DEF_PRECISION_INT
-                    : col->disp_precision );
+      precision=col->disp_precision; /* See description in unsigned types.*/
       break;
 
 
@@ -342,8 +352,7 @@ gal_tableintern_col_print_info(gal_data_t *col, int tableformat,
       fmt[0] = tableformat==GAL_TABLE_FORMAT_TXT ? 'd' : 'I';
       width=( col->disp_width<=0 ? GAL_TABLE_DEF_WIDTH_LINT
               : col->disp_width );
-      precision=( col->disp_precision<=0 ? GAL_TABLE_DEF_PRECISION_INT
-                  : col->disp_precision );
+      precision=col->disp_precision; /* See description in unsigned types.*/
       break;
 
 
@@ -351,18 +360,22 @@ gal_tableintern_col_print_info(gal_data_t *col, int tableformat,
     /* We need a default value (because in most cases, it won't be set. */
     case GAL_TYPE_FLOAT32:
     case GAL_TYPE_FLOAT64:
-      /* Set the format. */
-      switch(col->disp_fmt)
-        {
-        case GAL_TABLE_DISPLAY_FMT_FLOAT:
-          fmt[0] = tableformat==GAL_TABLE_FORMAT_TXT ? 'f' : 'F'; break;
-        case GAL_TABLE_DISPLAY_FMT_EXP:
-          fmt[0] = tableformat==GAL_TABLE_FORMAT_TXT ? 'e' : 'E'; break;
-        case GAL_TABLE_DISPLAY_FMT_GENERAL:
-          fmt[0] = tableformat==GAL_TABLE_FORMAT_TXT ? 'g' : 'E'; break;
-        default:
-          fmt[0] = tableformat==GAL_TABLE_FORMAT_TXT ? 'g' : 'E'; break;
-        }
+
+      /* Set the format. For FITS-ASCII, dealing with 'F' (fixed-point
+         notation) can create bad CFITSIO crashes if the total number of
+         digits becomes larger than the width. We won't have this problem
+         with 'E' (exponential) notation, so we'll just write all the
+         floating points in exponential notation. */
+      if(tableformat==GAL_TABLE_FORMAT_AFITS) fmt[0]='E';
+      else
+        switch(col->disp_fmt)
+          {
+          case GAL_TABLE_DISPLAY_FMT_FIXED:   fmt[0]='f'; break;
+          case GAL_TABLE_DISPLAY_FMT_EXP:     fmt[0]='e'; break;
+          case GAL_TABLE_DISPLAY_FMT_GENERAL: fmt[0]='g'; break;
+          default:  /* '%e' is the most conservative in plain-text:  */
+            fmt[0] = 'e'; break;  /* it is independent of the power. */
+          }
 
       /* Set the width and precision */
       switch(col->type)
@@ -370,22 +383,30 @@ gal_tableintern_col_print_info(gal_data_t *col, int tableformat,
         case GAL_TYPE_FLOAT32:
           width     = ( col->disp_width<=0
                         ? GAL_TABLE_DEF_WIDTH_FLT : col->disp_width );
-          precision = ( col->disp_precision<=0
-                        ? GAL_TABLE_DEF_PRECISION_FLT : col->disp_precision );
+          precision = ( col->disp_precision==GAL_BLANK_INT
+                        ? GAL_TABLE_DEF_PRECISION_FLT
+                        : col->disp_precision );
           break;
         case GAL_TYPE_FLOAT64:
           width     = ( col->disp_width<=0
                         ? GAL_TABLE_DEF_WIDTH_DBL : col->disp_width );
-
-          /* CFITSIO doesn't recognize the double precision defined here
-             for ASCII FITS tables. */
-          precision = ( col->disp_precision<=0
-                        ? ( tableformat==GAL_TABLE_FORMAT_TXT
-                            ? GAL_TABLE_DEF_PRECISION_DBL
-                            : GAL_TABLE_DEF_PRECISION_FLT )
+          precision = ( col->disp_precision==GAL_BLANK_INT
+                        ? GAL_TABLE_DEF_PRECISION_DBL
                         : col->disp_precision );
           break;
         }
+
+      /* In ASCII-FITS tables, the full number (including the sign, the
+         point, the 'E+123' should fit within the width. For example in
+         '-7.32384000000E+04' (with a width of 18 characters), besides the
+         digits after the decimal point, we need the following 8 characters
+         '-7.E+NNN'. Therefore the precision should not be more than 10. If
+         the precision is more than this, CFITSIO is going to crash. We
+         should therefore correct it here. To be safe, we'll also add
+         another integer and leave 9 characters of the width for
+         non-precision characters. */
+      if(tableformat==GAL_TABLE_FORMAT_AFITS && width-precision<9)
+        precision=width-9;
       break;
 
 
@@ -405,9 +426,9 @@ gal_tableintern_col_print_info(gal_data_t *col, int tableformat,
 
 
 /* Use the input 'blank' string and the input column to put the blank value
-   in the column's array. This function should later be generalized into a
-   function to read a string into a given data type (see
-   'gal_data_string_to_array_elem'). It is only here temporarily. */
+   in the column's array. If the string cannot be interpretted as a blank
+   of that type, then store it in the 'mmapname' element of the data
+   structure. */
 void
 gal_tableintern_read_blank(gal_data_t *col, char *blank)
 {
@@ -423,10 +444,16 @@ gal_tableintern_read_blank(gal_data_t *col, char *blank)
 
   /* Read the blank value as the given type. If successful, then
      'gal_data_string_to_type' will return 0. In that case, we need to
-     initialize the necessary paramters to read this data structure
-     correctly. */
-  if( !gal_type_from_string((void **)(&col->array), blank, col->type) )
+     initialize the necessary parameters to read this data structure
+     correctly. If it isn't successful, then  */
+  if( gal_type_from_string((void **)(&col->array), blank, col->type) )
     {
+      col->flag |= GAL_TABLEINTERN_FLAG_ARRAY_IS_BLANK_STRING;
+      gal_checkset_allocate_copy(blank, (char **)(&col->array));
+    }
+  else
+    {
+      col->flag=0;
       col->dsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, 1, 0, __func__,
                                       "col->dsize");
       col->dsize[0]=col->ndim=col->size=1;
