@@ -6,7 +6,7 @@ Original author:
      Sachin Kumar Singh <sachinkumarsingh092@gmail.com>
 Contributing author(s):
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
-Copyright (C) 2020, Free Software Foundation, Inc.
+Copyright (C) 2020-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -88,7 +88,6 @@ wcsdistortion_get_tpvparams(struct wcsprm *wcs, double cd[2][2],
   disseq=wcs->lin.disseq;
   keyp=disseq->dp;
 
-
   /* Fill the 2-times allocated CD array (cd[][]). Note that the required
     CD matix is extracted using the `gal_wcs_wrap_matrix` as a single
     allocated array (temp_cd[]), that is then used to fill cd[][]. */
@@ -110,13 +109,12 @@ wcsdistortion_get_tpvparams(struct wcsprm *wcs, double cd[2][2],
       /* For axis1. */
       if (keyp->j == 1)
         {
-          /* Ignore any missing keyvalues. */
-          if ( keyp->field == NULL ) continue;
+          if ( keyp->field[0] == '\0' ) continue;
           cp = strchr(keyp->field, '.') + 1;
           if (strncmp(cp, "TPV.", 4) != 0) continue;
           cp += 4;
 
-          sscanf(cp, "%ld", &index);
+          sscanf(cp, "%zu", &index);
           pv1[index]=disseq->dp[i].value.f;
 
           /* For a check
@@ -127,14 +125,12 @@ wcsdistortion_get_tpvparams(struct wcsprm *wcs, double cd[2][2],
       /* For axis2. */
       else if (keyp->j == 2)
         {
-          /* Ignore any missing keyvalues. */
-          if ( keyp->field == NULL ) continue;
-
+          if ( keyp->field[0] == '\0' ) continue;
           cp = strchr(keyp->field, '.') + 1;
           if (strncmp(cp, "TPV.", 4) != 0) continue;
           cp += 4;
 
-          sscanf(cp, "%ld", &index);
+          sscanf(cp, "%zu", &index);
 
           pv2[index]=disseq->dp[i].value.f;
 
@@ -207,14 +203,12 @@ wcsdistortion_get_sipparams(struct wcsprm *wcs, double cd[2][2],
       /* For axis1. */
       if (keyp->j == 1)
         {
-          /* Ignore any missing keyvalues. */
-          if ( keyp->field == NULL ) continue;
-
+          if ( keyp->field[0] == '\0' ) continue;
           cp = strchr(keyp->field, '.') + 1;
           if (strncmp(cp, "SIP.FWD.", 8) != 0) continue;
           cp += 8;
 
-          sscanf(cp, "%ld_%ld", &m, &n);
+          sscanf(cp, "%zu_%zu", &m, &n);
           a_coeff[m][n]=dispre->dp[i].value.f;
 
           /*For a check.
@@ -225,14 +219,12 @@ wcsdistortion_get_sipparams(struct wcsprm *wcs, double cd[2][2],
       /* For axis2. */
       else if (keyp->j == 2)
         {
-          /* Ignore any missing keyvalues. */
-          if ( keyp->field == NULL ) continue;
-
+          if ( keyp->field[0] == '\0' ) continue;
           cp = strchr(keyp->field, '.') + 1;
           if (strncmp(cp, "SIP.FWD.", 8) != 0) continue;
           cp += 8;
 
-          sscanf(cp, "%ld_%ld", &m, &n);
+          sscanf(cp, "%zu_%zu", &m, &n);
 
           b_coeff[m][n]=dispre->dp[i].value.f;
 
@@ -1446,7 +1438,13 @@ wcsdistortion_add_sipkeywords(struct wcsprm *wcs, size_t *fitsize,
   size_t ap_order=0, bp_order=0;
   size_t m, n, num=0, numkey=200;
   double ap_coeff[5][5], bp_coeff[5][5];
-  char *fullheader, fmt[50], sipkey[8], keyaxis[9], pcaxis[10];
+
+  /* The literal strings are a little longer than necessary because GCC
+     will complain/warn that size_t can be very long. Although it will
+     never happen that we need to write 10-decimal numbers in these, but to
+     have a clean build, we'll just set the literal space to be large
+     enough to avoid the warnings. */
+  char *fullheader, fmt[50], sipkey[50], keyaxis[50], pcaxis[50];
 
   /* Initialise the 2d matrices. */
   *nkeys = 0;
@@ -1532,7 +1530,7 @@ wcsdistortion_add_sipkeywords(struct wcsprm *wcs, size_t *fitsize,
         if(val != 0)
           {
             /* Make keywords */
-            sprintf(sipkey, "A_%ld_%ld", m, n);
+            sprintf(sipkey, "A_%zu_%zu", m, n);
             sprintf(fullheader+(FLEN_CARD-1)*num++, fmt, sipkey, val, "");
             a_order=wcsdistortion_max(a_order, wcsdistortion_max(m,n));
           }
@@ -1542,16 +1540,16 @@ wcsdistortion_add_sipkeywords(struct wcsprm *wcs, size_t *fitsize,
         if(val != 0)
           {
             /* Make keywords */
-            sprintf(sipkey, "B_%ld_%ld", m, n);
+            sprintf(sipkey, "B_%zu_%zu", m, n);
             sprintf(fullheader+(FLEN_CARD-1)*num++, fmt, sipkey, val, "");
             b_order=wcsdistortion_max(b_order, wcsdistortion_max(m,n));
           }
 
       }
 
-  sprintf(fullheader+(FLEN_CARD-1)*num++, "%-8s= %20ld%50s", "A_ORDER",
+  sprintf(fullheader+(FLEN_CARD-1)*num++, "%-8s= %20zu%50s", "A_ORDER",
           a_order, "");
-  sprintf(fullheader+(FLEN_CARD-1)*num++, "%-8s= %20ld%50s", "B_ORDER",
+  sprintf(fullheader+(FLEN_CARD-1)*num++, "%-8s= %20zu%50s", "B_ORDER",
           b_order, "");
 
   /* If reverse coefficients are required. */
@@ -1570,7 +1568,7 @@ wcsdistortion_add_sipkeywords(struct wcsprm *wcs, size_t *fitsize,
             if(val != 0)
               {
                 /* Make keywords */
-                sprintf(sipkey, "AP_%ld_%ld", m, n);
+                sprintf(sipkey, "AP_%zu_%zu", m, n);
                 sprintf(fullheader+(FLEN_CARD-1)*num++, fmt, sipkey,
                         val, "");
               }
@@ -1580,15 +1578,15 @@ wcsdistortion_add_sipkeywords(struct wcsprm *wcs, size_t *fitsize,
             if(val != 0)
               {
                 /* Make keywords */
-                sprintf(sipkey, "BP_%ld_%ld", m, n);
+                sprintf(sipkey, "BP_%zu_%zu", m, n);
                 sprintf(fullheader+(FLEN_CARD-1)*num++, fmt, sipkey,
                         val, "");
               }
           }
 
-      sprintf(fullheader+(FLEN_CARD-1)*num++, "%-8s= %20ld%50s", "AP_ORDER",
+      sprintf(fullheader+(FLEN_CARD-1)*num++, "%-8s= %20zu%50s", "AP_ORDER",
               ap_order, "");
-      sprintf(fullheader+(FLEN_CARD-1)*num++, "%-8s= %20ld%50s", "BP_ORDER",
+      sprintf(fullheader+(FLEN_CARD-1)*num++, "%-8s= %20zu%50s", "BP_ORDER",
               bp_order, "");
 
     }
@@ -1618,7 +1616,13 @@ wcsdistortion_add_pvkeywords(struct wcsprm *wcs, double *pv1,
   uint8_t i, j, k=0;
   int size = wcs->naxis;
   size_t m, n, num=0, numkey=100;
-  char *fullheader, fmt[50], pvkey[8], keyaxis[9], pcaxis[10];
+
+  /* The literal strings are a little longer than necessary because GCC
+     will complain/warn that size_t can be very long. Although it will
+     never happen that we need to write 10-decimal numbers in these, but to
+     have a clean build, we'll just set the literal space to be large
+     enough to avoid the warnings. */
+  char *fullheader, fmt[50], pvkey[50], keyaxis[50], pcaxis[50];
 
   /* Initialize values. */
   *nkeys = 0;
@@ -1704,7 +1708,7 @@ wcsdistortion_add_pvkeywords(struct wcsprm *wcs, double *pv1,
             if(val != 0)
               {
                 /* Make keywords */
-                sprintf(pvkey, "PV%ld_%ld", m, n);
+                sprintf(pvkey, "PV%zu_%zu", m, n);
                 sprintf(fullheader+(FLEN_CARD-1)*num++, fmt, pvkey,
                         val, "");
               }
@@ -1717,7 +1721,7 @@ wcsdistortion_add_pvkeywords(struct wcsprm *wcs, double *pv1,
             if(val != 0)
               {
                 /* Make keywords */
-                sprintf(pvkey, "PV%ld_%ld", m, n);
+                sprintf(pvkey, "PV%zu_%zu", m, n);
                 sprintf(fullheader+(FLEN_CARD-1)*num++, fmt, pvkey,
                         val, "");
               }
