@@ -6,7 +6,7 @@
 # Original author:
 #     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
-# Copyright (C) 2015-2021, Free Software Foundation, Inc.
+# Copyright (C) 2015-2023 Free Software Foundation, Inc.
 #
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
@@ -55,5 +55,5 @@ if [ ! -f $img      ]; then echo "$img does not exist.";   exit 77; fi
 # string. Such programs will execute the command if present and help in
 # debugging when the developer doesn't have access to the user's system.
 $check_with_program $execname $img --x --y --ra --dec --magnitude     \
-                              --upperlimitmag --sn --tableformat=txt  \
+                              --upperlimit-mag --sn --tableformat=txt  \
                               --clumpscat --output=objects-clumps.txt
